import os
import yaml
from .template import Template
from .errors import NoTemplateProvided, TemplateProvidedDoesntExist


class Examplerfile:
    """
    This class allows loading Examplerfiles, and generates premade Task Packages for API to use.
    """
    def __init__(self, playbook_path: str):
        """
        :param playbook_path str: A path to YAML file of Examplerfile.
        """
        with open(playbook_path, 'r', encoding='utf-8') as f:
            self.playbook = yaml.safe_load(f)

    def create_task(self, vars: dict, path: str) -> dict:
        """
        This method creates Task dictionaries.

        :param vars dict: Vairables to use with the Template
        :param path str: Path to save image/workflow pair to
        :return dict: Task object
        """
        if 'TEMPLATE' not in vars:
            raise NoTemplateProvided(path)
        if not os.path.isdir(f"templates/{vars['TEMPLATE']}"):
            raise TemplateProvidedDoesntExist(path, f"templates/{vars['TEMPLATE']}")

        newTemplate = Template(f"templates/{vars['TEMPLATE']}")
        task = {
            'workflow_api': newTemplate.generate_for_api(vars),
            'save_path_image': f'{path}.png',
            'workflow': newTemplate.generate_saveable_workflow(vars),
            'save_path_workflow': f'{path}.json'
        }

        return task

    def parse_playbook_recursive(self, playbook_layer: dict, path: str, variables: dict) -> list:
        """
        This method parses currently loaded examplerfile and returns list of tasks on this examplerfile.
        It is an recursive method, and must not be called directly!!!

        :param playbook_layer dict: Current layer of playbook.
        :param path str: Current path of playbook.
        :param variables dict: Current variables of playbook.
        :return list: List of dictionaries (Tasks)
        """
        # Step 1: Clone variables to not mess with the original ones
        playbook_layer_variables = {x: variables[x] for x in variables}

        # Step 2: Gather all 'global' variables into layer variables
        # Overwrite all preceding ones.
        for item in playbook_layer.get('vars', dict()):
            playbook_layer_variables[item] = playbook_layer['vars'][item]

        # Step 3: Generate all tasks in 'items'
        # Store them in resulting list.
        result = []
        if len(playbook_layer.get('items', [])) > 0:
            for item in playbook_layer['items']:
                item_variables = {x: playbook_layer_variables[x] for x in playbook_layer_variables}
                for var in playbook_layer['items'][item]:
                    item_variables[var] = playbook_layer['items'][item][var]

                result.append(self.create_task(item_variables, path + item))

        # Step 4: Iterate Play for every (children)/folder.
        for item in playbook_layer.get('children', []):
            result += self.parse_playbook_recursive(playbook_layer['children'][item], path + item + "/", playbook_layer_variables)

        return result

    def parse_playbook(self, output_path: str = './', **kwargs) -> tuple:
        """
        This method parses currently loaded examplerfile and returns list of tasks on this examplerfile.

        :param output_path str: Path to save tasks to.
        :param **kwargs: You can specify variables={ANYTHING: ANYTHING} to act as initial variables.
                         You can specify root_path="root" to act as where to start in examplebook
        :return tuple[dict]: Tuple of dictionaries, that describe every single Task possible.
        """
        starting_variables = kwargs.get('variables', dict())
        starting_root_point = kwargs.get('root_path', 'root')
        return tuple(self.parse_playbook_recursive(self.playbook[starting_root_point], output_path, starting_variables))
